=begin

Copyright 2011, Andreas Eisenbarth
All Rights Reserved

Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided that the above
copyright notice appear in all copies.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

Name:         LaunchUp.rb
Author:       Andreas Eisenbarth
Usage:        menu Plugins > 'LaunchUp - Tool Launcher'
              or create a shortcut, for example ctrl+space or ctrl+f
Description:  This Plugin adds a quick launcher to
                (1) Load plugins on the fly (if some plugins are not yet loaded or disabled)
                    requirement: index must contain a name and the path to the ruby file
                    Then the plugin is available at least from the menu/toolbars like normal, 
                    or it will be directly executed by (2).
                (2) Execute tools. 
                    requirement: (1), index must also contain the command of the tool
                    If necessary, the required plugin is also loaded on the fly.
Version:      0.1  
Date:         16.07.2011

=end



module LaunchUp
require 'sketchup.rb'
@dir = File.dirname(__FILE__) # this plugin's folder
@index_file = File.join(@dir,"index.txt") # index file
@plugins_dir = Sketchup.find_support_file("Plugins") # TODO: what about (more than one) loadpaths?
@windowTitlebarHeight = 0

@launchdlg = nil #FIXME: remove this after debugging


##
# main method, displays webdialog
#
def self.run
  if !@launchdlg
    # read index file
    return UI.messagebox("The index file could not be found.", MB_OK) if !File.exists?(@index_file)
    @index = IO.read(@index_file) # string of the index
    return (@index="[]";UI.messagebox("The index file is empty.", MB_OK)) if @index.empty?
    # create webdialog
    @launchdlg = UI::WebDialog.new("LaunchUp", false, false, 800, 200, 270, 50, true)
    html_path = File.join(@dir, "LaunchUp.htm")
    @launchdlg.set_file(html_path)
    @launchdlg.set_size(270,50)
    @launchdlg.show {
      @launchdlg.execute_script("index="+@index)
    }
    @launchdlg.add_action_callback("launch") {|@launchdlg, param|
      eval(@launchdlg.get_element_value("return-value"))
    }
    @launchdlg.add_action_callback("windowInnerHeight") {|@launchdlg, windowInnerHeight|
      @windowTitlebarHeight = 50-windowInnerHeight.to_f if @windowTitlebarHeight == 0
    }
    @launchdlg.add_action_callback("resizeHeight") {|@launchdlg, param|
      @launchdlg.set_size(270,@windowTitlebarHeight+param.to_f)
    }
    # for debugging
    @launchdlg.add_action_callback("puts") {|@launchdlg, param|
      puts param.to_s
    }
  # toggle webdialog. Useful for shortcuts
  elsif @launchdlg.visible?
    @launchdlg.bring_to_front
    @launchdlg.execute_script("$('combo-input').focus()")
    # @launchdlg.close
  else
    @launchdlg.show{
      @launchdlg.execute_script("index="+@index)
    }
  end
end #def run



##
# *Experimental*
#  Parse ruby files in order to extract if possible the name and command of tools
#
def self.build_index
  # @index is a json string
  # index is a hash, currently only needed in this method
  index = eval(@index.gsub(":","=>"))
  d = @plugins_dir
  # TODO: eventually add directory for disabled plugins, or _PLUGINS ...
  Dir.foreach(d){|f|
    path = File.join(d,f)
    # only continue with ruby files
    next if f == "."|| f == ".." || !FileTest.file?(path) || f[/\.rb/].nil?
    relpath = self.rel_path(path, @plugins_dir)
    # skip if file is already in index
    next if nil!=index.detect{|i| i["file"]==relpath}
    name = f.gsub(/\.rb$/,'')
    content = IO.read(path)
    # extract commands from UI:Command.new or menu.add_item or toolbar.add_item...
    commands = content.scan(/UI\:\:Command.new\s*\(\s*(?:'|")(.*?)(?:'|")\s*\)\s*\{\s*(.*?)\}/m)
    item = {"name" => name, "file" => relpath}
    # plugins that contain more than one command (many tools)
    if commands.length > 1
      item["tools"] = []
      commands.each{|c|
        item["tools"].push({"name" => c[0], "cmd" => c[1]}) 
      }
    # plugins with one command
    elsif commands.length == 1
      item["name"] = commands[0][0]
      item["cmd"] = commands[0][1]
    end
    index.push(item)
  }
  # create again json string from new index
  @index = index.inspect.gsub("=>", ":")
  # update index in webdialog
  @launchdlg.execute_script("index="+@index) if @launchdlg!=nil
  # save new index file
  @out = File.new(@index_file, "w")
  @out.puts @index
  @out.close
  # Question: should we only add files that are not yet in the index (check for index[i]["file"])? - Yes
  # How to remove plugins from index that are not anymore installed?
  # Performance?
  # Reliability? It won't work with many plugins, but with those where it works, could it cause errors?
end #def buildIndex



########## helper methods ##########



##
# Convert absolute file path into relative path, relative to
# - <base> directory or
# - next directory that is common with <file> given as second argument
#
def self.rel_path(abs,base=@plugins_dir)
  base=base.sub(/\/?$/,"/")
  i=0
  i+=1 until abs[i]!=base[i]
  i-=1 if i>0
  rel = abs.scan(/^.{0,#{i}}\/(.*$)/).to_s
  return (i==0)? abs : rel
end #def rel_path



end #module LaunchUp



if( not file_loaded?(File.basename(__FILE__)) )
  cmd_launchup = UI::Command.new("LaunchUp - Tool Launcher"){LaunchUp.run}
	UI.menu("Plugins").add_item(cmd_launchup)
end

file_loaded(File.basename(__FILE__))
